Queue

Struct Queue 

pub struct Queue<H, Cost>
where H: Handle, Cost: PartialOrd,
{ /* private fields */ }
Priorty queue implementation for mesh elements.

When you pop an element, the one with the lowest cost is removed from the queue and returned.

Implementations

Source

impl<H, Cost> Queue<H, Cost>
where H: Handle, Cost: PartialOrd,

pub fn new(num_items: usize) -> Self

Create a new heap with the number of elements.

For example, if the heap is meant to be used to queue up vertices of a mesh, provide the number of vertices in that mesh.

pub fn clear(&mut self)

Clear the queue.

pub fn len(&self) -> usize

Number of items currently in the queue.

pub fn is_empty(&self) -> bool

Check if this queue is empty.

pub fn insert(&mut self, val: H, cost: Cost)

Insert the given handle val and cost into the heap.

If the handle is already present in the heap, it will be updated to have the new cost.

pub fn remove(&mut self, val: H)

Remove an element from the queue, if it is present.

pub fn pop(&mut self) -> Option<(H, Cost)>

Remove the item from the front of the queue and return it.