VH

Struct VH 

pub struct VH { /* private fields */ }
Vertex handle.

Implementations

Source

impl VH

pub fn halfedge(self, mesh: &impl HasTopology) -> Option<HH>

pub fn is_valid(self, mesh: &impl HasTopology) -> bool

Check if this vertex is valid for the mesh.

The index has to be less than the number of vertices in the mesh.

pub fn is_manifold(self, mesh: &impl HasIterators) -> bool

Check if this vertex is manifold.

A vertex is manifold if it has at most 1 outgoing halfedge.

   .......|     .......|.......     ....\     /...
   .......|     .......|.......     .....\   /....
   .......|     .......|.......     ......\ /.....
   -------v     -------v-------     -------v------
   .......|     .......|.......     ....../ \.....
   .......|     .......|.......     ...../   \....
   .......|     .......|.......     ..../     \...
   Manifold     Manifold            Not manifold
pub fn is_boundary(self, mesh: &impl HasTopology) -> bool

Check if this vertex is on the boundary of the mesh.

pub fn valence(self, mesh: &impl HasIterators) -> usize

The number of edges incident on this vertex.