pub struct Queue<H, Cost>where
H: Handle,
Cost: PartialOrd,{ /* private fields */ }Expand description
Priorty queue implementation for mesh elements.
When you pop an element, the one with the lowest cost is removed from the queue and returned.
Implementations
Sourceimpl<H, Cost> Queue<H, Cost>where
H: Handle,
Cost: PartialOrd,
impl<H, Cost> Queue<H, Cost>where
H: Handle,
Cost: PartialOrd,
Sourcepub fn new(num_items: usize) -> Self
pub fn new(num_items: usize) -> Self
Create a new heap with the number of elements.
For example, if the heap is meant to be used to queue up vertices of a mesh, provide the number of vertices in that mesh.