pub struct VH { /* private fields */ }Expand description
Vertex handle.
Implementations
Sourceimpl VH
impl VH
pub fn halfedge(self, mesh: &impl HasTopology) -> Option<HH>
Sourcepub fn is_valid(self, mesh: &impl HasTopology) -> bool
pub fn is_valid(self, mesh: &impl HasTopology) -> bool
Check if this vertex is valid for the mesh.
The index has to be less than the number of vertices in the mesh.
Sourcepub fn is_manifold(self, mesh: &impl HasIterators) -> bool
pub fn is_manifold(self, mesh: &impl HasIterators) -> bool
Check if this vertex is manifold.
A vertex is manifold if it has at most 1 outgoing halfedge.
.......| .......|....... ....\ /...
.......| .......|....... .....\ /....
.......| .......|....... ......\ /.....
-------v -------v------- -------v------
.......| .......|....... ....../ \.....
.......| .......|....... ...../ \....
.......| .......|....... ..../ \...
Manifold Manifold Not manifoldSourcepub fn is_boundary(self, mesh: &impl HasTopology) -> bool
pub fn is_boundary(self, mesh: &impl HasTopology) -> bool
Check if this vertex is on the boundary of the mesh.
Sourcepub fn valence(self, mesh: &impl HasIterators) -> usize
pub fn valence(self, mesh: &impl HasIterators) -> usize
The number of edges incident on this vertex.